# -*- coding: utf8 -*-

from __future__ import with_statement

import sys

import infoauth
import mod_mysql
import herrams

MAILSERVER = "smtp.gmail.com"

MAIL_MSG= u"""\
¡Hola %s!

Hay nuevos comentarios en el post '%s' del blog de Facundo!

Podés verlos accediendo al siguiente enlace:

    http://www.taniquetil.com.ar/plog/post/1/%d

Saludos!

- bdvbot
"""

class LastID(object):
    """Guarda en disco el último id.

    Ante cualquier duda, devuelve el último de la BDD. Puede forzarse a
    trabajar desde un id puntual.
    """
    _filename = "lastid.txt"

    def __init__(self, bdd, fromid=None):
        if fromid is not None:
            self.lastid = int(fromid)
            self._save(self.lastid)
        else:
            try:
                self.lastid = self._get_from_file()
            except:
                # no desde el archivo
                self.lastid = self._get_from_db(bdd)
                self._save(self.lastid)

    def save_max(self, data):
        """Guarda el idlastid al archivo."""
        data = list(data)
        if not data:
            # nothing new to save
            return

        lastid = max(data)
        self._save(lastid)

    def _save(self, lastid):
        with open(self._filename, "w") as fh:
            fh.write("%d\n" % lastid)

    def _get_from_file(self):
        """Levanta el id del archivo."""
        with open(self._filename) as fh:
            data = fh.read()
        return int(data.strip())

    def _get_from_db(self, bdd):
        """Levanta la info de la base de datos."""
        r = bdd.accion('select max(id) from plog_articles_comments;')
        return int(r[0][0])


def get_info_per_article(bdd, artid):
    comments = bdd.accion('''select user_name, user_email
                             from plog_articles_comments where article_id=%d
                             order by date;''' % artid)

    # el último vio los anteriores no tiene que recibir el mail
    comments = comments[:-1]

    # get only the people with mail
    data = [(x[0].decode("latin1"), x[1]) for x in comments if x[1].strip()]

    # get the title
    title = bdd.accion('''select topic from plog_articles_text
                          where id=%d;''' % artid)
    title = title[0][0].decode("latin1")
    return title, data


def send_mail(artid, title, destination, mail_sender):
    name, mail = destination
    subject = u"Hay nuevos comentarios en Bitácora de Vuelo!"
    body = MAIL_MSG % (name, title, artid)

    subject = u"Comentario en: " + title
    mail = [mail, "facundo@taniquetil.com.ar"] # DEBUG
    mail_sender.envia(mail, subject, body)


def main(host, fromid):
    dbusr, dbclv, mailusr, mailclv = infoauth.load("comentarios.auth")
    bdd = mod_mysql.DataBase(host, dbusr, dbclv, "plog")
    lid = LastID(bdd, fromid)

    data = bdd.accion("select id, article_id from plog_articles_comments "
                      "where id > %d and status=0;" % lid.lastid)

    # grabamos el último id
    lid.save_max(x[0] for x in data)

    mail_sender = herrams.MailSender(MAILSERVER, mailusr,
                                     auth=(mailusr, mailclv),
                                     expl_from="Bitácora de Vuelo")

    # get the articles, send mails per article
    articles = set(x[1] for x in data)
    for artid in articles:
        title, destinations = get_info_per_article(bdd, artid)
        for destin in set(destinations):
            send_mail(artid, title, destin, mail_sender)


if __name__ == "__main__":
    if len(sys.argv) not in (2, 3):
        print "Usar: comentarios.py <host> [from_id]"
        sys.exit()

    host = sys.argv[1]
    if len(sys.argv) == 3:
        fromid = sys.argv[2]
    else:
        fromid = None
    main(host, fromid)
